@@ -70,7 +70,7 @@ testInternalSelectGet ()
   fd_set rs;
   fd_set ws;
   fd_set es;
-  MHD_socket max;
+  int maxposixs; /* Max socket number unused on W32 */
   int running;
   struct CURLMsg *msg;
   time_t start;
@@ -134,11 +134,11 @@ testInternalSelectGet ()
   start = time (NULL);
   while ((time (NULL) - start < 5) && (multi != NULL))
     {
-      max = 0;
+      maxposixs = -1;
       FD_ZERO (&rs);
       FD_ZERO (&ws);
       FD_ZERO (&es);
-      mret = curl_multi_fdset (multi, &rs, &ws, &es, &max);
+      mret = curl_multi_fdset (multi, &rs, &ws, &es, &maxposixs);
       if (mret != CURLM_OK)
         {
           curl_multi_remove_handle (multi, c);
@@ -149,7 +149,7 @@ testInternalSelectGet ()
         }
       tv.tv_sec = 0;
       tv.tv_usec = 1000;
-      select (max + 1, &rs, &ws, &es, &tv);
+      select (maxposixs + 1, &rs, &ws, &es, &tv);
       curl_multi_perform (multi, &running);
       if (running == 0)
         {
