@@ -150,9 +150,14 @@ ServeOneRequest(void *param)
       MHD_SYS_select_ (max + 1, &rs, &ws, &es, &tv);
       MHD_run (d);
     }
+  fd = MHD_quiesce_daemon (d);
+  if (MHD_INVALID_SOCKET == fd)
+    {
+      MHD_stop_daemon (d);
+      return "MHD_quiesce_daemon() failed in ServeOneRequest()";
+    }
   MHD_stop_daemon (d);
-  MHD_socket_close_(fd);
-  return NULL;
+  return done ? NULL : "Requests was not served by ServeOneRequest()";
 }
 
 
@@ -229,6 +234,14 @@ testGet (int type, int pool_count, int poll_flag)
   }
 
   fd = MHD_quiesce_daemon (d);
+  if (MHD_INVALID_SOCKET == fd)
+    {
+      fprintf (stderr,
+               "MHD_quiesce_daemon failed.\n");
+      curl_easy_cleanup (c);
+      MHD_stop_daemon (d);
+      return 2;
+    }
   if (0 != pthread_create(&thrd, NULL, &ServeOneRequest, (void*)(intptr_t) fd))
     {
       fprintf (stderr, "pthread_create failed\n");
@@ -411,9 +424,16 @@ testExternalGet ()
       if (i == 0) {
         /* quiesce the daemon on the 1st iteration, so the 2nd should fail */
         fd = MHD_quiesce_daemon(d);
-	if (MHD_INVALID_SOCKET == fd)
-	  abort ();
-	MHD_socket_close_ (fd);
+        if (MHD_INVALID_SOCKET == fd)
+          {
+            fprintf (stderr,
+                     "MHD_quiesce_daemon failed.\n");
+            curl_multi_remove_handle (multi, c);
+            curl_multi_cleanup (multi);
+            curl_easy_cleanup (c);
+            MHD_stop_daemon (d);
+            return 2;
+          }
         c = setupCURL (&cbc);
         multi = curl_multi_init ();
         mret = curl_multi_add_handle (multi, c);
@@ -434,6 +454,7 @@ testExternalGet ()
       curl_multi_cleanup (multi);
     }
   MHD_stop_daemon (d);
+  MHD_socket_close_ (fd);
   if (cbc.pos != strlen ("/hello_world"))
     return 8192;
   if (0 != strncmp ("/hello_world", cbc.buf, strlen ("/hello_world")))
